@@ -6,7 +6,6 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"net/url"
 	"os"
 	"os/exec"
 	"time"
@@ -133,10 +132,6 @@ func setupFromTemplate(ctx context.Context) (context.Context, error) {
 		return ctx, nil
 	}
 
-	if _, err := url.Parse(from); err != nil {
-		return ctx, fmt.Errorf("invalid URL: `%s'. Expected https:// git repo", from)
-	}
-
 	entries, err := os.ReadDir(".")
 	if err != nil {
 		return ctx, fmt.Errorf("failed to read directory: %w", err)
