@@ -1,13 +1,20 @@
 package config
 
 import (
+	"context"
+	"errors"
+	"io/fs"
 	"sync"
+	"time"
 
+	"github.com/fsnotify/fsnotify"
 	"github.com/spf13/pflag"
 
 	"github.com/superfly/fly-go/tokens"
 	"github.com/superfly/flyctl/internal/env"
+	"github.com/superfly/flyctl/internal/flag/flagctx"
 	"github.com/superfly/flyctl/internal/flag/flagnames"
+	"github.com/superfly/flyctl/internal/task"
 )
 
 const (
@@ -47,7 +54,12 @@ const (
 //
 // Instances of Config are safe for concurrent use.
 type Config struct {
-	mu sync.RWMutex
+	mu   sync.RWMutex
+	path string
+
+	watchOnce sync.Once
+	watchErr  error
+	subs      map[chan *Config]struct{}
 
 	// APIBaseURL denotes the base URL of the API.
 	APIBaseURL string
@@ -93,22 +105,34 @@ type Config struct {
 	MetricsToken string
 }
 
-// New returns a new instance of Config populated with default values.
-func New() *Config {
-	return &Config{
+func Load(ctx context.Context, path string) (*Config, error) {
+	cfg := &Config{
 		APIBaseURL:     defaultAPIBaseURL,
 		FlapsBaseURL:   defaultFlapsBaseURL,
 		RegistryHost:   defaultRegistryHost,
 		MetricsBaseURL: defaultMetricsBaseURL,
 		Tokens:         new(tokens.Tokens),
 	}
+
+	// Apply config from the config file, if it exists
+	if err := cfg.applyFile(path); err != nil && !errors.Is(err, fs.ErrNotExist) {
+		return nil, err
+	}
+
+	// Apply config from the environment, overriding anything from the file
+	cfg.applyEnv()
+
+	// Finally, apply command line options, overriding any previous setting
+	cfg.applyFlags(flagctx.FromContext(ctx))
+
+	return cfg, nil
 }
 
-// ApplyEnv sets the properties of cfg which may be set via environment
+// applyEnv sets the properties of cfg which may be set via environment
 // variables to the values these variables contain.
 //
-// ApplyEnv does not change the dirty state of config.
-func (cfg *Config) ApplyEnv() {
+// applyEnv does not change the dirty state of config.
+func (cfg *Config) applyEnv() {
 	cfg.mu.Lock()
 	defer cfg.mu.Unlock()
 
@@ -131,12 +155,14 @@ func (cfg *Config) ApplyEnv() {
 	cfg.SendMetrics = env.IsTruthy(SendMetricsEnvKey) || cfg.SendMetrics
 }
 
-// ApplyFile sets the properties of cfg which may be set via configuration file
+// applyFile sets the properties of cfg which may be set via configuration file
 // to the values the file at the given path contains.
-func (cfg *Config) ApplyFile(path string) (err error) {
+func (cfg *Config) applyFile(path string) (err error) {
 	cfg.mu.Lock()
 	defer cfg.mu.Unlock()
 
+	cfg.path = path
+
 	var w struct {
 		AccessToken  string `yaml:"access_token"`
 		MetricsToken string `yaml:"metrics_token"`
@@ -158,9 +184,9 @@ func (cfg *Config) ApplyFile(path string) (err error) {
 	return
 }
 
-// ApplyFlags sets the properties of cfg which may be set via command line flags
+// applyFlags sets the properties of cfg which may be set via command line flags
 // to the values the flags of the given FlagSet may contain.
-func (cfg *Config) ApplyFlags(fs *pflag.FlagSet) {
+func (cfg *Config) applyFlags(fs *pflag.FlagSet) {
 	cfg.mu.Lock()
 	defer cfg.mu.Unlock()
 
@@ -188,6 +214,141 @@ func (cfg *Config) MetricsBaseURLIsProduction() bool {
 	return cfg.MetricsBaseURL == defaultMetricsBaseURL
 }
 
+func (cfg *Config) Watch(ctx context.Context) (chan *Config, error) {
+	cfg.watchOnce.Do(func() {
+		watch, err := fsnotify.NewWatcher()
+		if err != nil {
+			cfg.watchErr = err
+			return
+		}
+
+		if err := watch.Add(cfg.path); err != nil {
+			cfg.watchErr = err
+			return
+		}
+
+		cfg.subs = make(map[chan *Config]struct{})
+
+		task.FromContext(ctx).Run(func(ctx context.Context) {
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
+
+			cleanupDone := make(chan struct{})
+			defer func() { <-cleanupDone }()
+
+			go func() {
+				defer close(cleanupDone)
+
+				<-ctx.Done()
+
+				cfg.mu.Lock()
+				defer cfg.mu.Unlock()
+
+				cfg.watchErr = errors.Join(cfg.watchErr, ctx.Err(), watch.Close())
+
+				for sub := range cfg.subs {
+					close(sub)
+				}
+				cfg.subs = nil
+			}()
+
+			var (
+				notifyCtx        context.Context
+				cancelNotify     context.CancelFunc  = func() {}
+				cancelLastNotify *context.CancelFunc = &cancelNotify
+			)
+			defer func() { (*cancelLastNotify)() }()
+
+			for {
+				select {
+				case e, open := <-watch.Events:
+					if !open {
+						return
+					}
+
+					if !e.Has(fsnotify.Write) {
+						continue
+					}
+
+					// Debounce change notifications: notifySubs sleeps for 50ms
+					// before notifying subs. If we get another change before
+					// that, we preempt the previous notification attempt. This
+					// is necessary because we receive multiple notifications
+					// for a single config change on windows and the first event
+					// fires before the change is available to be read.
+					(*cancelLastNotify)()
+					notifyCtx, cancelNotify = context.WithCancel(ctx)
+					cancelLastNotify = &cancelNotify
+
+					go cfg.notifySubs(notifyCtx)
+				case err := <-watch.Errors:
+					cfg.mu.Lock()
+					defer cfg.mu.Unlock()
+
+					cfg.watchErr = errors.Join(cfg.watchErr, err)
+
+					return
+				case <-ctx.Done():
+					return
+				}
+			}
+		})
+	})
+
+	cfg.mu.Lock()
+	defer cfg.mu.Unlock()
+
+	if cfg.watchErr != nil {
+		return nil, cfg.watchErr
+	}
+
+	sub := make(chan *Config)
+	cfg.subs[sub] = struct{}{}
+
+	return sub, nil
+}
+
+func (cfg *Config) Unwatch(sub chan *Config) {
+	cfg.mu.Lock()
+	defer cfg.mu.Unlock()
+
+	if cfg.subs != nil {
+		delete(cfg.subs, sub)
+		close(sub)
+	}
+}
+
+func (cfg *Config) notifySubs(ctx context.Context) {
+	// sleep for 50ms to facilitate debouncing (described above)
+	select {
+	case <-ctx.Done():
+		return
+	case <-time.After(50 * time.Millisecond):
+	}
+
+	newCfg, err := Load(ctx, cfg.path)
+	if err != nil {
+		return
+	}
+
+	cfg.mu.RLock()
+	defer cfg.mu.RUnlock()
+
+	// just in case we have a slow subscriber
+	timer := time.NewTimer(100 * time.Millisecond)
+	defer timer.Stop()
+
+	for sub := range cfg.subs {
+		select {
+		case sub <- newCfg:
+		case <-timer.C:
+			return
+		case <-ctx.Done():
+			return
+		}
+	}
+}
+
 func applyStringFlags(fs *pflag.FlagSet, flags map[string]*string) {
 	for name, dst := range flags {
 		if !fs.Changed(name) {
