@@ -3,23 +3,21 @@ package scanner
 import (
 	"bufio"
 	"encoding/base64"
+	"encoding/json"
+	"errors"
 	"fmt"
-	"github.com/superfly/flyctl/helpers"
+	"io/fs"
+	"log"
 	"os"
 	"os/exec"
 	"regexp"
 	"strconv"
 	"strings"
+
+	"github.com/superfly/flyctl/helpers"
+	"github.com/superfly/flyctl/internal/command/launch/plan"
 )
 
-type ComposerLock struct {
-	Platform PhpVersion `json:"platform,omitempty"`
-}
-
-type PhpVersion struct {
-	Version string `json:"php"`
-}
-
 // setup Laravel with a sqlite database
 func configureLaravel(sourceDir string, config *ScannerConfig) (*SourceInfo, error) {
 	// Laravel projects contain the `artisan` command
@@ -27,8 +25,6 @@ func configureLaravel(sourceDir string, config *ScannerConfig) (*SourceInfo, err
 		return nil, nil
 	}
 
-	files := templates("templates/laravel")
-
 	s := &SourceInfo{
 		Env: map[string]string{
 			"APP_ENV":               "production",
@@ -39,7 +35,6 @@ func configureLaravel(sourceDir string, config *ScannerConfig) (*SourceInfo, err
 			"SESSION_SECURE_COOKIE": "true",
 		},
 		Family: "Laravel",
-		Files:  files,
 		Port:   8080,
 		Secrets: []Secret{
 			{
@@ -54,6 +49,7 @@ func configureLaravel(sourceDir string, config *ScannerConfig) (*SourceInfo, err
 		},
 		SkipDatabase:   true,
 		ConsoleCommand: "php /var/www/html/artisan tinker",
+		Callback:       LaravelCallback,
 	}
 
 	phpVersion, err := extractPhpVersion()
@@ -80,6 +76,93 @@ func configureLaravel(sourceDir string, config *ScannerConfig) (*SourceInfo, err
 	return s, nil
 }
 
+func LaravelCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) error {
+	// create temporary fly.toml for merge purposes
+	flyToml := "fly.toml"
+	_, err := os.Stat(flyToml)
+	if os.IsNotExist(err) {
+		// create a fly.toml consisting only of an app name
+		contents := fmt.Sprintf("app = \"%s\"\n", appName)
+		err := os.WriteFile(flyToml, []byte(contents), 0644)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		// inform caller of the presence of this file
+		srcInfo.MergeConfig = &MergeConfigStruct{
+			Name:      flyToml,
+			Temporary: true,
+		}
+	}
+
+	// generate Dockerfile if it doesn't already exist
+	dockerfileExists := true
+	_, err = os.Stat("Dockerfile")
+	if errors.Is(err, fs.ErrNotExist) {
+		dockerfileExists = false
+	}
+
+	// check first to see if the package is already installed
+	installed := false
+
+	data, err := os.ReadFile("composer.json")
+	if err == nil {
+		var composerJson map[string]interface{}
+		err = json.Unmarshal(data, &composerJson)
+		if err == nil {
+			// check for the package in the composer.json
+			require, ok := composerJson["require"].(map[string]interface{})
+			if ok && require["fly-apps/dockerfile-laravel"] != nil {
+				installed = true
+			}
+
+			requireDev, ok := composerJson["require-dev"].(map[string]interface{})
+			if ok && requireDev["fly-apps/dockerfile-laravel"] != nil {
+				installed = true
+			}
+		}
+	}
+
+	// install fly-apps/dockerfile-laravel if it's not already installed
+	if !installed {
+		args := []string{"composer", "require", "--dev", "fly-apps/dockerfile-laravel"}
+		fmt.Printf("installing: %s\n", strings.Join(args, " "))
+		cmd := exec.Command(args[0], args[1:]...)
+		cmd.Stdin = os.Stdin
+		cmd.Stdout = os.Stdout
+		cmd.Stderr = os.Stderr
+
+		if err := cmd.Run(); err != nil && !dockerfileExists {
+			return fmt.Errorf("Dockerfile doesn't exist and failed to install fly-apps/dockerfile-laravel: %w", err)
+		}
+	}
+
+	args := []string{"vendor/bin/dockerfile-laravel", "generate"}
+	if dockerfileExists {
+		args = append(args, "--skip")
+	}
+	fmt.Printf("Running: %s\n", strings.Join(args, " "))
+	cmd := exec.Command(args[0], args[1:]...)
+	cmd.Stdin = os.Stdin
+	cmd.Stdout = os.Stdout
+	cmd.Stderr = os.Stderr
+
+	if err := cmd.Run(); err != nil {
+		return fmt.Errorf("failed to generate Dockerfile: %w", err)
+	}
+
+	// provide some advice
+	srcInfo.DeployDocs += fmt.Sprintf(`
+If you need custom packages installed, or have problems with your deployment
+build, you may need to edit the Dockerfile for app-specific changes. If you
+need help, please post on https://community.fly.io.
+
+Now: run 'fly deploy' to deploy your %s app.
+`, srcInfo.Family)
+
+	return nil
+}
+
 func extractPhpVersion() (string, error) {
 	/* Example Output:
 	PHP 8.1.8 (cli) (built: Jul  8 2022 10:58:31) (NTS)
