@@ -3,7 +3,6 @@ package scanner
 import (
 	"fmt"
 	"io/fs"
-	"log"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -86,10 +85,14 @@ func configureRails(sourceDir string, config *ScannerConfig) (*SourceInfo, error
 		// mysql
 		s.DatabaseDesired = DatabaseKindMySQL
 		s.SkipDatabase = false
-	} else {
+	} else if !checksPass(sourceDir, fileExists("Dockerfile")) || checksPass(sourceDir, dirContains("Dockerfile", "libpq-dev", "postgres")) {
 		// postgresql
 		s.DatabaseDesired = DatabaseKindPostgres
 		s.SkipDatabase = false
+	} else {
+		// sqlite
+		s.DatabaseDesired = DatabaseKindSqlite
+		s.SkipDatabase = true
 	}
 
 	// enable redis if there are any action cable / anycable channels
@@ -155,7 +158,6 @@ func configureRails(sourceDir string, config *ScannerConfig) (*SourceInfo, error
 		}
 	}
 
-	s.SkipDeploy = true
 	s.DeployDocs = `
 Your Rails app is prepared for deployment.
 
@@ -188,28 +190,51 @@ Once ready: run 'fly deploy' to deploy your Rails app.
 
 func RailsCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) error {
 	// install dockerfile-rails gem, if not already included
+	writable := false
 	gemfile, err := os.ReadFile("Gemfile")
 	if err != nil {
 		panic(err)
 	} else if !strings.Contains(string(gemfile), "dockerfile-rails") {
-		cmd := exec.Command(bundle, "add", "dockerfile-rails",
-			"--optimistic", "--group", "development", "--skip-install")
-		cmd.Stdin = nil
-		cmd.Stdout = os.Stdout
-		cmd.Stderr = os.Stderr
-
-		if err := cmd.Run(); err != nil {
-			return errors.Wrap(err, "Failed to add dockerfile-rails gem, exiting")
+		// check for writable gem installation directory
+		out, err := exec.Command("gem", "environment").Output()
+		if err == nil {
+			regexp := regexp.MustCompile(`INSTALLATION DIRECTORY: (.*)\n`)
+			for _, match := range regexp.FindAllStringSubmatch(string(out), -1) {
+				// Testing to see if a directory is writable is OS dependent, so
+				// we use a brute force method: attempt it and see if it works.
+				file, err := os.CreateTemp(match[1], ".flyctl.probe")
+				if err == nil {
+					writable = true
+					file.Close()
+					defer os.Remove(file.Name())
+				}
+			}
 		}
 
-		cmd = exec.Command(bundle, "install", "--quiet")
-		cmd.Stdin = nil
-		cmd.Stdout = os.Stdout
-		cmd.Stderr = os.Stderr
+		// install dockerfile-rails gem if the gem installation directory is writable
+		if writable {
+			cmd := exec.Command(bundle, "add", "dockerfile-rails",
+				"--optimistic", "--group", "development", "--skip-install")
+			cmd.Stdin = nil
+			cmd.Stdout = os.Stdout
+			cmd.Stderr = os.Stderr
 
-		if err := cmd.Run(); err != nil {
-			return errors.Wrap(err, "Failed to install dockerfile-rails gem, exiting")
+			if err := cmd.Run(); err != nil {
+				return errors.Wrap(err, "Failed to add dockerfile-rails gem, exiting")
+			}
+
+			cmd = exec.Command(bundle, "install", "--quiet")
+			cmd.Stdin = nil
+			cmd.Stdout = os.Stdout
+			cmd.Stderr = os.Stderr
+
+			if err := cmd.Run(); err != nil {
+				return errors.Wrap(err, "Failed to install dockerfile-rails gem, exiting")
+			}
 		}
+	} else {
+		// proceed as if the gem installation directory is writable
+		writable = true
 	}
 
 	// ensure Gemfile.lock includes the x86_64-linux platform
@@ -230,7 +255,7 @@ func RailsCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) e
 		// "touch" fly.toml
 		file, err := os.Create(flyToml)
 		if err != nil {
-			log.Fatal(err)
+			return errors.Wrap(err, "Failed to create fly.toml")
 		}
 		file.Close()
 
@@ -249,6 +274,10 @@ func RailsCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) e
 	_, err = os.Stat("Dockerfile")
 	if !errors.Is(err, fs.ErrNotExist) {
 		args = append(args, "--skip")
+
+		if !writable {
+			return errors.Wrap(err, "No Dockerfile found and unable to install dockerfile-rails gem")
+		}
 	}
 
 	// add postgres
@@ -262,7 +291,7 @@ func RailsCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) e
 	}
 
 	// run command
-	fmt.Printf("installing: %s\n", strings.Join(args[:], " "))
+	fmt.Printf("installing: %s\n", strings.Join(args, " "))
 	cmd := exec.Command(ruby, args...)
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
@@ -291,27 +320,6 @@ func RailsCallback(appName string, srcInfo *SourceInfo, plan *plan.LaunchPlan) e
 		}
 	}
 
-	// extract workdir
-	workdir := "$"
-	re = regexp.MustCompile(`(?m).*^WORKDIR\s+(?P<dir>/\S+)`)
-	m = re.FindStringSubmatch(string(dockerfile))
-
-	for i, name := range re.SubexpNames() {
-		if len(m) > 0 && name == "dir" {
-			workdir = m[i]
-		}
-	}
-
-	// add Statics if workdir is found and doesn't contain a variable reference
-	if !strings.Contains(workdir, "$") {
-		srcInfo.Statics = []Static{
-			{
-				GuestPath: workdir + "/public",
-				UrlPrefix: "/",
-			},
-		}
-	}
-
 	// add HealthCheck (if found)
 	srcInfo.HttpCheckPath = <-healthcheck_channel
 	if srcInfo.HttpCheckPath != "" {
